/*
 * addonmanager.cpp
 *
 *  Created on: 26.02.2009
 *      Author: darkstar
 */

#include "addonmanager.h"
#include "debug.h"

/* AddOn */

AddOn::AddOn()
    :	QObject()
{
    DGUARDMETHOD;
    qAddOnManager.registerAddOn(this);
}

AddOn::~AddOn()
{
    DGUARDMETHOD;
    qAddOnManager.unregisterAddOn(this);
}

/* AddOnManager */

AddOnManager &AddOnManager::singleton()
{
    static AddOnManager instance;
    return instance;
}

AddOnManager::AddOnManager()
{
#ifdef QT4
    addOns_ = new QList<AddOn *>();
#else
    addOns_ = new QList<AddOn>();
#endif
}

AddOnManager::~AddOnManager()
{
    DGUARDMETHODTIMED;
    for (int i = addOns_->count() - 1; i >= 0; --i)
        unregisterAddOn(addOns_->at(i));

    delete addOns_;
}

void AddOnManager::registerAddOn(AddOn *addOn)
{
    DGUARDMETHODF("%p", addOn);
    addOns_->append(addOn);
    addOn->addOnManager_ = this;
}

bool AddOnManager::unregisterAddOn(AddOn *addOn)
{
    DGUARDMETHODF("%p", addOn);
    addOn->addOnManager_ = 0;
#ifdef QT4
    return addOns_->removeAll(addOn);
#else
    return addOns_->removeRef(addOn);
#endif
}

void AddOnManager::hookMainWindow(Quasar *mainWindow)
{
    DGUARDMETHODTIMED;
    for (int i = 0; i < addOns_->count(); ++i)
         addOns_->at(i)->hookMainWindow(mainWindow);
}

void AddOnManager::hookPlayListView(PlayListView *playList)
{
    DGUARDMETHODTIMED;
    for (int i = 0; i < addOns_->count(); ++i)
        addOns_->at(i)->hookPlayListView(playList);
}

void AddOnManager::hookConfigurationDialog(ConfigurationDialog *cd)
{
    DGUARDMETHODTIMED;
    for (int i = 0; i < addOns_->count(); ++i)
        addOns_->at(i)->hookConfigurationDialog(cd);
}

void AddOnManager::writeToConfig(Config &cf)
{
    DGUARDMETHODTIMED;
    for (int i = 0; i < addOns_->count(); ++i)
        addOns_->at(i)->writeToConfig(cf);
}

void AddOnManager::readFromConfig(Config &cf)
{
    DGUARDMETHODTIMED;
    for (int i = 0; i < addOns_->count(); ++i)
        addOns_->at(i)->readFromConfig(cf);
}

void AddOnManager::init()
{
    DGUARDMETHODTIMED;
    for (int i = 0; i < addOns_->count(); ++i)
        addOns_->at(i)->init();
}

void AddOnManager::deinit()
{
    DGUARDMETHODTIMED;
    for (int i = 0; i < addOns_->count(); ++i)
        addOns_->at(i)->deinit();
}
